import os,sys

NAME='rack'

try:
	RUBYPATH = os.environ['UWSGICONFIG_RUBYPATH']
except:
	RUBYPATH = 'ruby'

rbconfig = 'Config'

version = os.popen(RUBYPATH + " -e \"print RUBY_VERSION\"").read().rstrip()
v = version.split('.')

GCC_LIST = ['rack_plugin', 'rack_api']

if (v[0] == '1' and v[1] == '9') or v[0] >= '2':
    CFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['CFLAGS']\"").read().rstrip().split()
    CFLAGS.append('-DRUBY19')
    CFLAGS.append('-Wno-unused-parameter')
    rbconfig = 'RbConfig'	 
else:
    CFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['CFLAGS']\"" % rbconfig).read().rstrip().split()

includedir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['rubyhdrdir']\"" % rbconfig).read().rstrip()
if includedir == 'nil':
    includedir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['archdir']\"" % rbconfig).read().rstrip()
    CFLAGS.append('-I' + includedir)
else:
    CFLAGS.append('-I' + includedir)
    archdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['archdir']\"" % rbconfig).read().rstrip()
    arch = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['arch']\"" % rbconfig).read().rstrip()
    archdir2 = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['rubyarchhdrdir']\"" % rbconfig).read().rstrip()
    CFLAGS.append('-I' + archdir)
    CFLAGS.append('-I' + archdir + '/' + arch)
    CFLAGS.append('-I' + includedir + '/' + arch)
    if archdir2:
        CFLAGS.append('-I' + archdir2)

LDFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['LDFLAGS']\"" % rbconfig).read().rstrip().split()
libpath = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['libdir']\"" % rbconfig).read().rstrip()

has_shared = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['ENABLE_SHARED']\"" % rbconfig).read().rstrip()

LIBS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['LIBS']\"" % rbconfig).read().rstrip().split()

if has_shared == 'yes':
    LDFLAGS.append('-L' + libpath )
    os.environ['LD_RUN_PATH'] = libpath
    LIBS.append(os.popen(RUBYPATH + " -e \"require 'rbconfig';print '-l' + %s::CONFIG['RUBY_SO_NAME']\"" % rbconfig).read().rstrip())
else:
    rubylibdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['rubylibdir']\"").read().rstrip()
    rubyarchdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['archdir']\"").read().rstrip()
    # detect Heroku system
    heroku = False
    if rubylibdir.startswith('/tmp/build_'):
        heroku = True
        rubylibdir = '/app/' + '/'.join(rubylibdir.split('/')[3:])
    if rubyarchdir.startswith('/tmp/build_'):
        heroku = True
        rubyarchdir = '/app/' + '/'.join(rubyarchdir.split('/')[3:])
    if heroku:
        CFLAGS.append('-DUWSGI_RUBY_HEROKU')
    CFLAGS.append('-DUWSGI_RUBY_LIBDIR="\\"%s\\""' % rubylibdir)
    CFLAGS.append('-DUWSGI_RUBY_ARCHDIR="\\"%s\\""' % rubyarchdir)
    GCC_LIST.append("%s/%s" % (libpath, os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['LIBRUBY_A']\"" % rbconfig).read().rstrip()))




import os,sys

NAME='ruby19'

try:
	RUBYPATH = os.environ['UWSGICONFIG_RUBYPATH']
except:
	RUBYPATH = 'ruby'

rbconfig = 'Config'

version = os.popen(RUBYPATH + " -e \"print RUBY_VERSION\"").read().rstrip()
v = version.split('.')

GCC_LIST = ['../rack/rack_plugin', '../rack/rack_api']

if v[0] == '1' and v[1] == '9':
    CFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print RbConfig::CONFIG['CFLAGS']\"").read().rstrip().split()
    CFLAGS.append('-DRUBY19')
    CFLAGS.append('-Wno-unused-parameter')
    rbconfig = 'RbConfig'	 
else:
    CFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['CFLAGS']\"" % rbconfig).read().rstrip().split()

includedir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['rubyhdrdir']\"" % rbconfig).read().rstrip()
if includedir == 'nil':
    includedir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['archdir']\"" % rbconfig).read().rstrip()
    CFLAGS.append('-I' + includedir)
else:
    CFLAGS.append('-I' + includedir)
    archdir = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['archdir']\"" % rbconfig).read().rstrip()
    arch = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['arch']\"" % rbconfig).read().rstrip()
    CFLAGS.append('-I' + archdir)
    CFLAGS.append('-I' + archdir + '/' + arch)
    CFLAGS.append('-I' + includedir + '/' + arch)

CFLAGS.append('-Drack_plugin=ruby19_plugin')

LDFLAGS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['LDFLAGS']\"" % rbconfig).read().rstrip().split()

libpath = os.popen(RUBYPATH + " -e \"require 'rbconfig';print %s::CONFIG['libdir']\"" % rbconfig).read().rstrip()
LDFLAGS.append('-L' + libpath )
os.environ['LD_RUN_PATH'] = libpath
LIBS = os.popen(RUBYPATH + " -e \"require 'rbconfig';print '-l' + %s::CONFIG['RUBY_SO_NAME']\"" % rbconfig).read().rstrip().split()




import os

NAME='emperor_pg'
CFLAGS = os.popen('pg_config --cflags').read().rstrip().split()
CFLAGS.append('-I' + os.popen('pg_config --includedir').read().rstrip())
LDFLAGS = os.popen('pg_config --ldflags').read().rstrip().split()
LIBS = ['-L' + os.popen('pg_config --libdir').read().rstrip(), '-lpq']

GCC_LIST = ['emperor_pg']



import mmap
import os
import struct
import array
import time
import sys
from optparse import OptionParser

class Cache:

    def __init__(self, filename, cache_slots, block_size=65536, sample_sleep=1):

        self.block_size = block_size
        self.cache_slots = cache_slots

        self.key_size = 2048
        self.item_size = 2 + 2 + 4 + 8 + 8 + 8 + 8 + 8 + self.key_size
        self.block_size_start = self.item_size * self.cache_slots

        fd = os.open(filename, os.O_RDONLY)
        self.cache_store = mmap.mmap(fd, 0, mmap.MAP_SHARED, mmap.PROT_READ)

        self.sample_sleep = sample_sleep
        self.samples = 0
        self.history = []
        self.cache_full = 0
        self.cache_empty = 0
        self.cache_items = 0
        self.block_sizes = 0

    def store_read_item_block(self, position):
        pos = self.cache_store.tell()
        # uwsgi cache stores cache entries first and then the blocks
        self.cache_store.seek(self.block_size_start + (position * self.block_size))
        buf = self.cache_store.read(self.block_size)
        self.cache_store.seek(pos)
        return buf

    def store_read_item(self, position):
        buf = self.cache_store.read(self.item_size)
        fields = struct.unpack_from('@HHIQQQQQ2048c', buf)
        key = array.array('c', fields[8:self.key_size+8]).tostring().rstrip('\x00')

        if [x for x in key if x!= '\x00']:
            buf = self.store_read_item_block(position)
            value = array.array('c', buf).tostring().rstrip('\x00')
        else:
            value = ''
        return (position, key, value, len(value))

    def read(self):
        data = [self.store_read_item(i) for i in range(self.cache_slots)]
        self.cache_store.seek(0)
        self.update_stats(data)
        if self.sample_sleep:
            time.sleep(self.sample_sleep)
        return data
 
    def update_stats(self, data):
        # data is a list of (position, key, value, len(value)) tuples
        items = len([1 for x in data if x[3] > 0])
        self.cache_items += items
        full, empty = items == self.cache_slots, items == 0
        if full:
            self.cache_full += 1
        if empty:
            self.cache_empty += 1
        self.samples += 1
        block_sizes = sum([x[3] for x in data])
        self.block_sizes += block_sizes
        self.history.append({'full': full, 'empty': empty, 'data': data, \
            'items': items, 'block_sizes': block_sizes})

    def dump(self):
        return {
            'samples': self.samples,
            'history': self.history,
            'cache_slots': self.cache_slots,
            'sample_sleep': self.sample_sleep, 
            'cache_empty': self.cache_empty,
            'cache_full': self.cache_full,
            'cache_items': self.cache_items,
            'block_sizes': self.block_sizes,
         }

    def show_dump(self):
        d = self.dump()
        print
        print "Recorded %d samples (%d second(s) sleep between samples)" % \
            (d['samples'], d['sample_sleep'])
        print "Cache empty %d times, full %d times, %.2f items on average" % \
            (d['cache_empty'], d['cache_full'], d['cache_items'] / d['samples'])
        print "Block size average size: %d bytes" % \
            (d['block_sizes'] / d['cache_items'] * 8)
        print "Data in cache average: %d bytes" % \
            (d['block_sizes'] / d['samples'] * 8)

def main(options):
    cache = Cache(options.cache_store, options.cache_slots, options.block_size,
        options.sleep_time)
    print "Recording..."
    while True:
        try:
            data = cache.read()
        except KeyboardInterrupt:
            cache.show_dump()
            sys.exit(0)

if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option("-s", "--cache-slots", dest="cache_slots", type="int",
        help="Slots available in the cache, uwsgi cache option")
    parser.add_option("-c", "--cache-store", dest="cache_store", default="uwsgi.cache",
        help="The filename of the cache store, uwsgi cache-store option. Default: uwsgi.cache")
    parser.add_option("-b", "--block-size", dest="block_size", default=65536, type="int",
        help="The size of the cache block, uwsgi cache-blocksize option. Default: 65536")
    parser.add_option("-t", "--sleep-time", dest="sleep_time", default=1, type="int",
        help="The time to sleep between each sample. Default: 1")

    (options, args) = parser.parse_args()
    if not options.cache_slots:
        parser.error('Option -s / --cache-slots is mandatory')
    main(options)



import os,sys

LUAINC = os.environ.get('UWSGICONFIG_LUAINC')
LUALIB = os.environ.get('UWSGICONFIG_LUALIB')
LUALIBPATH = os.environ.get('UWSGICONFIG_LUALIBPATH')
LUAPC = os.environ.get('UWSGICONFIG_LUAPC', 'lua5.1')

# we LUAINC/LUALIB/LUALIBPATH override the LUAPC for backwards compat
if LUAINC:
	CFLAGS = ['-I%s' % LUAINC]
else:
	try:
		CFLAGS = os.popen('pkg-config --cflags %s' % LUAPC).read().rstrip().split()
	except:
		CFLAGS = ['-I/usr/include/lua5.1']

if LUALIB:
	LIBS = ['-l%s' % LUALIB]
else:
	try:
		LIBS = os.popen('pkg-config --libs %s' % LUAPC).read().rstrip().split() 
	except:
		LIBS = ['-llua5.1']

if LUALIBPATH:
	LDFLAGS = ['-L%s' % LUALIBPATH]
else:
	LDFLAGS = []

NAME='lua'
GCC_LIST = ['lua_plugin']





import os,sys

from distutils import sysconfig

def get_python_version():
    version = sysconfig.get_config_var('VERSION')
    try:
        version = version + sys.abiflags
    except:
        pass
    return version

NAME='python'
GCC_LIST = ['python_plugin', 'pyutils', 'pyloader', 'wsgi_handlers', 'wsgi_headers', 'wsgi_subhandler', 'web3_subhandler', 'pump_subhandler', 'gil', 'uwsgi_pymodule', 'profiler', 'symimporter', 'tracebacker', 'raw']

CFLAGS = ['-I' + sysconfig.get_python_inc(), '-I' + sysconfig.get_python_inc(plat_specific=True) ] 
LDFLAGS = []

if not 'UWSGI_PYTHON_NOLIB' in os.environ:
    LIBS = sysconfig.get_config_var('LIBS').split() + sysconfig.get_config_var('SYSLIBS').split()
    # check if it is a non-shared build (but please, add --enable-shared to your python's ./configure script)
    if not sysconfig.get_config_var('Py_ENABLE_SHARED'):
        libdir = sysconfig.get_config_var('LIBPL')
        # libdir does not exists, try to get it from the venv
        version = get_python_version()
        if not os.path.exists(libdir):
            libdir = '%s/lib/python%s/config' % (sys.prefix, version)
        # try skipping abiflag
        if not os.path.exists(libdir) and version.endswith('m'):
            version = version[:-1]
            libdir = '%s/lib/python%s/config' % (sys.prefix, version)
        # try 3.x style config dir
        if not os.path.exists(libdir):
            libdir = '%s/lib/python%s/config-%s' % (sys.prefix, version, get_python_version())

        # get cpu type
        uname = os.uname()
        if uname[4].startswith('arm'):
            libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LIBRARY'))
            if not os.path.exists(libpath): 
                libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LDLIBRARY'))
        else:
            libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LDLIBRARY'))
            if not os.path.exists(libpath): 
                libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LIBRARY'))
        if not os.path.exists(libpath): 
            libpath = '%s/libpython%s.a' % (libdir, version)
        LIBS.append(libpath)
        # hack for messy linkers/compilers
        if '-lutil' in LIBS:
            LIBS.append('-lutil')
    else:
        try:
            LDFLAGS.append("-L%s" % sysconfig.get_config_var('LIBDIR'))
            os.environ['LD_RUN_PATH'] = "%s" % (sysconfig.get_config_var('LIBDIR'))
        except:
            LDFLAGS.append("-L%s/lib" % sysconfig.PREFIX)
            os.environ['LD_RUN_PATH'] = "%s/lib" % sysconfig.PREFIX

        LIBS.append('-lpython%s' % get_python_version())
else:
    LIBS = []



import os

NAME='alarm_speech'

uwsgi_os = os.uname()[0]

LDFLAGS = []
if uwsgi_os == "Darwin":
    CFLAGS = []
    LIBS = ['-framework appkit']
else:
    CFLAGS = ['-I /usr/include/GNUstep']
    LIBS = []

GCC_LIST = ['alarm_speech.m']



import django
from django.core.management.base import BaseCommand
from django.conf import settings
import os
import sys

class Command(BaseCommand):
    help = "Runs this project as a uWSGI application. Requires the uwsgi binary in system path."

    http_port = '8000'
    socket_addr = None

    def handle(self, *args, **options):
        for arg in args:
            k,v = arg.split('=')
            if k == 'http':
                if self.http_port:
                   self.http_port = v
            elif k == 'socket':
                self.http_port = None
                self.socket_addr = v

        # load http and python plugin: first the specific version, otherwise try with the generic one
        if self.http_port:
            os.environ['UWSGI_PLUGINS'] = 'http,python%d%d:python' % (sys.version_info[0], sys.version_info[1])
        else:
            os.environ['UWSGI_PLUGINS'] = 'python%d%d:python' % (sys.version_info[0], sys.version_info[1])

        # load the Django WSGI handler
        os.environ['UWSGI_MODULE'] = 'django.core.handlers.wsgi:WSGIHandler()'
        # DJANGO settings
        if options['settings']:
            os.environ['DJANGO_SETTINGS_MODULE'] = options['settings']
        else:
            os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

        # bind the http server to the default port
        if self.http_port:
            os.environ['UWSGI_HTTP'] = ':%s' % self.http_port
        elif self.socket_addr:
            os.environ['UWSGI_SOCKET'] = self.socket_addr

	# map admin static files
	os.environ['UWSGI_STATIC_MAP'] = '%s=%s' % (settings.ADMIN_MEDIA_PREFIX, os.path.join(django.__path__[0], 'contrib', 'admin', 'media'))
        # remove sockets/pidfile at exit
        os.environ['UWSGI_VACUUM'] = '1'
        # retrieve/set the PythonHome
        os.environ['UWSGI_PYHOME'] = sys.prefix
        # increase buffer size a bit
        os.environ['UWSGI_BUFFER_SIZE'] = '8192'
        # add threads for concurrency
        os.environ['UWSGI_THREADS'] = '8'
        # enable the master process
        os.environ['UWSGI_MASTER'] = '1'
        # use uWSGI python module aliasing to fix the PYTHONPATH
        os.environ['UWSGI_PYMODULE_ALIAS'] = '%s=./' % os.path.basename(os.getcwd())
        # exec the uwsgi binary
        os.execvp('uwsgi', ('uwsgi',))

    def usage(self, subcomand):
        return r"""
run this project on the uWSGI server

  http=PORT		run the embedded http server on port PORT
  socket=ADDR		bind the uwsgi server on address ADDR (this will disable the http server)
        """



from picasso import *

def home(req):
  if not req["session"].get("user"):
    return redirect("/login")
  return "<h1>Welcome back, %s!</h1>" % req["session"]["user"]

def login(req):
  return "<form method='post'><input type='submit' name='foo' value='login' /></form>"

def login_post(req):
  print req
  req["session"]["user"] = "James"
  return redirect("/")

routes = setup_routes(
  GET("/", home),
  GET("/login", login),
  POST("/login", login_post),
  routing.not_found("<h1>Not Found</h1>"))

app = setup_app(routes)



import uwsgi
import os

def xsendfile(e, sr):
    sr('200 OK', [('Content-Type', 'image/png'), ('X-Sendfile', os.path.abspath('logo_uWSGI.png'))])
    return b''

def serve_logo(e, sr):
    sr('200 OK', [('Content-Type', 'image/png')])
    return uwsgi.sendfile('logo_uWSGI.png')

def serve_config(e, sr):
    sr('200 OK', [('Content-Type', 'text/html')])
    for opt in uwsgi.opt.keys():
        body = "{opt} = {optvalue}<br/>".format(opt=opt, optvalue=uwsgi.opt[opt].decode('ascii'))
        yield bytes(body.encode('ascii'))

routes = {}
routes['/xsendfile'] = xsendfile
routes['/logo'] = serve_logo
routes['/config'] = serve_config

def application(env, start_response):

    if env['PATH_INFO'] in routes:
        return routes[env['PATH_INFO']](env, start_response)

    start_response('200 OK', [('Content-Type', 'text/html')])

    body = """
<img src="/logo"/> version {version}<br/>
<hr size="1"/>

Configuration<br/>
<iframe src="/config"></iframe><br/>

<br/>

    """.format(version=uwsgi.version.decode('ascii'))

    return bytes(body.encode('ascii'))






